@@ -171,63 +171,78 @@ void gnutls_transport_get_ptr2(gnutls_session_t session,
   * you should wait for an EOF from the peer.
   * GNUTLS_SHUT_WR sends an alert containing a close request.
   *
+  * Note that not all implementations will properly terminate a TLS connection.
+  * Some of them, usually for performance reasons, will terminate only the 
+  * underlying transport layer, thus causing a transmission error to the peer.
+  * This error cannot be distinguished from a malicious party prematurely terminating
+  * the session, thus this behavior is not recommended.
+  *
   * This function may also return GNUTLS_E_AGAIN or GNUTLS_E_INTERRUPTED; cf.
   * gnutls_record_get_direction().
   *
   **/
-int gnutls_bye(gnutls_session_t session, gnutls_close_request_t how)
+int
+gnutls_bye (gnutls_session session, gnutls_close_request how)
 {
-    int ret = 0;
+  int ret = 0;
 
-    switch (STATE) {
+  switch (STATE)
+    {
     case STATE0:
     case STATE60:
-	ret = _gnutls_io_write_flush(session);
-	STATE = STATE60;
-	if (ret < 0) {
-	    gnutls_assert();
-	    return ret;
+      ret = _gnutls_io_write_flush (session);
+      STATE = STATE60;
+      if (ret < 0)
+	{
+	  gnutls_assert ();
+	  return ret;
 	}
 
     case STATE61:
-	ret =
-	    gnutls_alert_send(session, GNUTLS_AL_WARNING,
-			      GNUTLS_A_CLOSE_NOTIFY);
-	STATE = STATE61;
-	if (ret < 0) {
-	    gnutls_assert();
-	    return ret;
+      ret =
+	gnutls_alert_send (session, GNUTLS_AL_WARNING, GNUTLS_A_CLOSE_NOTIFY);
+      STATE = STATE61;
+      if (ret < 0)
+	{
+	  gnutls_assert ();
+	  return ret;
 	}
 
     case STATE62:
-	if (how == GNUTLS_SHUT_RDWR) {
-	    do {
-	        _gnutls_io_clear_peeked_data( session);
-	        ret = _gnutls_recv_int(session, GNUTLS_ALERT, -1, NULL, 0);
-            } while( ret == GNUTLS_E_GOT_APPLICATION_DATA);
+      STATE = STATE62;
+      if (how == GNUTLS_SHUT_RDWR)
+	{
+	  do
+	    {
+	      _gnutls_io_clear_peeked_data (session);
+	      ret = _gnutls_recv_int (session, GNUTLS_ALERT, -1, NULL, 0);
+	    }
+	  while (ret == GNUTLS_E_GOT_APPLICATION_DATA);
 
-	    if (ret >= 0)
-		session->internals.may_not_read = 1;
+	  if (ret >= 0)
+	    session->internals.may_not_read = 1;
 
-	    if (ret < 0) {
-	        gnutls_assert();
-	        return ret;
-  	    }
+	  if (ret < 0)
+	    {
+	      gnutls_assert ();
+	      return ret;
+	    }
 	}
-	STATE = STATE62;
+      STATE = STATE62;
 
-	break;
+      break;
     default:
-	gnutls_assert();
-	return GNUTLS_E_INTERNAL_ERROR;
+      gnutls_assert ();
+      return GNUTLS_E_INTERNAL_ERROR;
     }
 
-    STATE = STATE0;
+  STATE = STATE0;
 
-    session->internals.may_not_write = 1;
-    return 0;
+  session->internals.may_not_write = 1;
+  return 0;
 }
 
+
 inline static void session_invalidate(gnutls_session_t session)
 {
     session->internals.valid_connection = VALID_FALSE;
