@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
  * USA
  *
  */
@@ -755,7 +755,7 @@ int _gnutls_x509_decode_octet_string( const char* string_type,
     opaque* output, size_t* output_size)
 {
 ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
-int result;
+int result, tmp_output_size;
 char strname[64];
  
         if (string_type == NULL)
@@ -780,8 +780,10 @@ char strname[64];
 	    goto cleanup;
 	}
 
+        tmp_output_size = *output_size;
+	result = asn1_read_value(c2, "", output, &tmp_output_size);
+	*output_size = tmp_output_size;
 
-	result = asn1_read_value(c2, "", output, output_size);
 	if (result != ASN1_SUCCESS) {
 	    gnutls_assert();
 	    result = _gnutls_asn2err(result);
