@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002, 2003, 2004, 2005 Free Software Foundation
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006 Free Software Foundation
  *
  * Author: Nikos Mavroyanopoulos
  *
@@ -344,7 +344,7 @@ _gnutls_asn1_get_structure_xml(ASN1_TYPE structure,
 
 	if (p->type == TYPE_BIT_STRING) {
 	    len2 = -1;
-	    len = _asn1_get_length_der(p->value, &len2);
+	    len = _asn1_get_length_der(p->value, p->value_len, &len2);
 	    snprintf(tmp, sizeof(tmp), " length=\"%i\"",
 		     (len - 1) * 8 - (p->value[len2]));
 	    STR_APPEND(tmp);
@@ -374,7 +374,7 @@ _gnutls_asn1_get_structure_xml(ASN1_TYPE structure,
 	    case TYPE_INTEGER:
 		if (value) {
 		    len2 = -1;
-		    len = _asn1_get_length_der(value, &len2);
+		    len = _asn1_get_length_der(value, p->value_len, &len2);
 
 		    for (k = 0; k < len; k++) {
 			snprintf(tmp, sizeof(tmp),
@@ -387,7 +387,7 @@ _gnutls_asn1_get_structure_xml(ASN1_TYPE structure,
 	    case TYPE_ENUMERATED:
 		if (value) {
 		    len2 = -1;
-		    len = _asn1_get_length_der(value, &len2);
+		    len = _asn1_get_length_der(value, p->value_len, &len2);
 
 		    for (k = 0; k < len; k++) {
 			snprintf(tmp, sizeof(tmp),
@@ -412,7 +412,7 @@ _gnutls_asn1_get_structure_xml(ASN1_TYPE structure,
 	    case TYPE_BIT_STRING:
 		if (value) {
 		    len2 = -1;
-		    len = _asn1_get_length_der(value, &len2);
+		    len = _asn1_get_length_der(value, p->value_len, &len2);
 
 		    for (k = 1; k < len; k++) {
 			snprintf(tmp, sizeof(tmp),
@@ -424,7 +424,7 @@ _gnutls_asn1_get_structure_xml(ASN1_TYPE structure,
 	    case TYPE_OCTET_STRING:
 		if (value) {
 		    len2 = -1;
-		    len = _asn1_get_length_der(value, &len2);
+		    len = _asn1_get_length_der(value, p->value_len, &len2);
 		    for (k = 0; k < len; k++) {
 			snprintf(tmp, sizeof(tmp),
 				 "%02X", (value)[k + len2]);
@@ -440,7 +440,7 @@ _gnutls_asn1_get_structure_xml(ASN1_TYPE structure,
 		if (!p->down) {
 		    if (value) {
 			len3 = -1;
-			len2 = _asn1_get_length_der(value, &len3);
+			len2 = _asn1_get_length_der(value, p->value_len, &len3);
 			for (k = 0; k < len2; k++) {
 			    snprintf(tmp, sizeof(tmp),
 				     "%02X", (value)[k + len3]);
@@ -456,10 +456,10 @@ _gnutls_asn1_get_structure_xml(ASN1_TYPE structure,
 			up->left && up->left->value &&
 			up->type & CONST_DEFINED_BY &&
 			type_field(up->left->type) == TYPE_OBJECT_ID) {
+		      len2 = _asn1_get_length_der(up->value,
+						  up->value_len, &len3);
 
-			len2 = _asn1_get_length_der(up->value, &len3);
-
-			if (len2 > 0 && strcmp(p->name, "type") == 0) {
+		      if (len2 > 0 && strcmp(p->name, "type") == 0) {
 			    size_t tmp_len = sizeof(tmp);
 			    ret =
 				_gnutls_x509_oid_data2string(up->left->
